# -*- coding: utf-8 -*-
#
# File: testPlone.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

#
# Base TestCase for rendezvous
#

import os, sys, code
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

##code-section module-header #fill in your manual code here
##/code-section module-header

from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
from Products.rendezvous.config import PRODUCT_DEPENDENCIES
from Products.rendezvous.config import DEPENDENCIES

# Add common dependencies
DEPENDENCIES.append('Archetypes')
PRODUCT_DEPENDENCIES.append('MimetypesRegistry')
PRODUCT_DEPENDENCIES.append('PortalTransforms')
PRODUCT_DEPENDENCIES.append('rendezvous')

# Install all (product-) dependencies, install them too
for dependency in PRODUCT_DEPENDENCIES + DEPENDENCIES:
    ZopeTestCase.installProduct(dependency)

ZopeTestCase.installProduct('rendezvous')

PRODUCTS = list()
PRODUCTS += DEPENDENCIES
PRODUCTS.append('rendezvous')

testcase = PloneTestCase.PloneTestCase
##code-section module-before-plone-site-setup #fill in your manual code here
##/code-section module-before-plone-site-setup

PloneTestCase.setupPloneSite(products=PRODUCTS)

class testPlone(testcase):
    """Base TestCase for rendezvous."""

    ##code-section class-header_testPlone #fill in your manual code here
    ##/code-section class-header_testPlone

    # Commented out for now, it gets blasted at the moment anyway.
    # Place it in the protected section if you need it.
    #def afterSetup(self):
    #    """
    #    """
    #    pass

    def interact(self, locals=None):
        """Provides an interactive shell aka console inside your testcase.

        It looks exact like in a doctestcase and you can copy and paste
        code from the shell into your doctest. The locals in the testcase are
        available, becasue you are in the testcase.

        In your testcase or doctest you can invoke the shell at any point by
        calling::

            >>> self.interact( locals() )

        locals -- passed to InteractiveInterpreter.__init__()
        """
        savestdout = sys.stdout
        sys.stdout = sys.stderr
        sys.stderr.write('='*70)
        console = code.InteractiveConsole(locals)
        console.interact("""
ZopeTestCase Interactive Console
(c) BlueDynamics Alliance, Austria - 2005

Note: You have the same locals available as in your test-case.
""")
        sys.stdout.write('\nend of ZopeTestCase Interactive Console session\n')
        sys.stdout.write('='*70+'\n')
        sys.stdout = savestdout


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testPlone))
    return suite

##code-section module-footer #fill in your manual code here
##/code-section module-footer

if __name__ == '__main__':
    framework()





# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'


import logging
logger = logging.getLogger('rendezvous: setuphandlers')
from Products.rendezvous.config import PROJECTNAME
from Products.rendezvous.config import DEPENDENCIES
import os
from Products.CMFCore.utils import getToolByName
import transaction
##code-section HEAD
##/code-section HEAD

def isNotrendezvousProfile(context):
    return context.readDataFile("rendezvous_marker.txt") is None

def setupHideTypesFromSearch(context):
    """hide selected classes in the search form"""
    if isNotrendezvousProfile(context): return 
    # XXX use https://svn.plone.org/svn/collective/DIYPloneStyle/trunk/profiles/default/properties.xml
    site = context.getSite()
    portalProperties = getToolByName(site, 'portal_properties')
    siteProperties = getattr(portalProperties, 'site_properties')
    for klass in ['RDV_Proposition']:
        propertyid = 'types_not_searched'
        lines = list(siteProperties.getProperty(propertyid) or [])
        if klass not in lines:
            lines.append(klass)
            siteProperties.manage_changeProperties(**{propertyid: lines})

def setupHideMetaTypesFromNavigations(context):
    """hide selected classes in the search form"""
    if isNotrendezvousProfile(context): return 
    # XXX use https://svn.plone.org/svn/collective/DIYPloneStyle/trunk/profiles/default/properties.xml
    site = context.getSite()
    portalProperties = getToolByName(site, 'portal_properties')
    navtreeProperties = getattr(portalProperties, 'navtree_properties')
    for klass in ['RDV_Proposition']:
        propertyid = 'metaTypesNotToList'
        lines = list(navtreeProperties.getProperty(propertyid) or [])
        if klass not in lines:
            lines.append(klass)
            navtreeProperties.manage_changeProperties(**{propertyid: lines})



def updateRoleMappings(context):
    """after workflow changed update the roles mapping. this is like pressing
    the button 'Update Security Setting' and portal_workflow"""
    if isNotrendezvousProfile(context): return 
    wft = getToolByName(context.getSite(), 'portal_workflow')
    wft.updateRoleMappings()

def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotrendezvousProfile(context): return
    site = context.getSite()



##code-section FOOT
##/code-section FOOT



# -*- coding: utf-8 -*-
#
# File: RDV_RendezVousEdit.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

##code-section module-header #fill in your manual code here
from plone.portlets.interfaces import IPortletRenderer
from plone.app.portlets.portlets import calendar
from plone.portlets.interfaces import IPortletManager
from zope.component import getUtility, getMultiAdapter
from Products.CMFCore.utils import getToolByName
from plone.app.portlets.portlets.calendar import Renderer as CalRenderer
from plone.memoize.compress import xhtml_compress
from Acquisition import aq_inner
from Products.rendezvous.browser.RDV_RendezVousUtility import RDV_RendezVousUtility

class _RDV_Calendar(CalRenderer):
    """overload the calendar for removing events
    """
    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.view = view
        self.__parent__ = view

    def render(self):
        return xhtml_compress(self._template())

    def getDatesForCalendar(self):
        context = aq_inner(self.context)
        year = self.year
        month = self.month
        weeks_ = self.calendar.getWeeksList(month, year)
        selected_dates = self.view.getSelectedDates()
        weeks = []
        for week_ in weeks_:
            week = []
            for daynumber in week_:
                day = {}
                week.append(day)
                day['day'] = daynumber
                if daynumber == 0:
                    continue
                day['is_today'] = self.isToday(daynumber)
                day['date_string'] = '%d-%0.2d-%0.2d' % (year, month, daynumber)
                if day['date_string'] in selected_dates:
                    day['selected'] = True
                else:
                    day['selected'] = False
            weeks.append(week)
        return weeks
##/code-section module-header

from zope import interface
from zope import component
from Products.CMFPlone import utils
from Products.Five import BrowserView
from zope.interface import implements
from Products.rendezvous.content.RDV_RendezVous import RDV_RendezVous
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class RDV_RendezVousEdit(BrowserView):
    """
    """

    ##code-section class-header_RDV_RendezVousEdit #fill in your manual code here
    ##/code-section class-header_RDV_RendezVousEdit

    def getSelectedDates(self):
        return RDV_RendezVousUtility.getSelectedDates(self)

    def toggleSelectedDate(self, selected_date):
        return RDV_RendezVousUtility.toggleSelectedDate(self, selected_date)

    def getFormatedDates(self):
        return [self.context.toLocalizedTime(date) for date in sorted(self.getSelectedDates())]

    def __getattr__(self, key):
        try:
            return self.__dict__[key]
        except KeyError:
            return getattr(self.renderer, key)



    def __init__(self, context, request):
        super(RDV_RendezVousEdit, self).__init__(context, request)

        date = self.request.get('rdvdate', None)
        if date:
            self.toggleSelectedDate(date)

        self.renderer = _RDV_Calendar(context, request, self)
        self.portal_catalog = getToolByName(self.context, 'portal_catalog')
        self.renderer.update()


##code-section module-footer #fill in your manual code here
##/code-section module-footer





# -*- coding: utf-8 -*-
#
# File: RDV_PropositionsEdit.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

##code-section module-header #fill in your manual code here
from Products.rendezvous.browser.RDV_RendezVousUtility import RDV_RendezVousUtility
##/code-section module-header

from zope import interface
from zope import component
from Products.CMFPlone import utils
from Products.Five import BrowserView
from zope.interface import implements
from Products.rendezvous.content.RDV_RendezVous import RDV_RendezVous
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class RDV_PropositionsEdit(BrowserView):
    """
    """

    ##code-section class-header_RDV_PropositionsEdit #fill in your manual code here
    NB_COLUMNS = 5
    ##/code-section class-header_RDV_PropositionsEdit

    def getNbColumns(self):
        """Return the number of columns
        """
        propositionsbydates = RDV_RendezVousUtility.getPropositionsByDates(self)
        try:
            uid = self.context.aq_inner.UID()
            nb = self.request.SESSION['rendezvous']['nb_columns'][uid]
        except KeyError:
            nb = self.NB_COLUMNS
        for date, propositions in propositionsbydates.items():
            nb = max(len(propositions), nb)
        return nb

    def incNbColumns(self):
        uid = self.context.aq_inner.UID()
        session_rendezvous = self.request.SESSION.get('rendezvous', {})
        if not 'nb_columns' in session_rendezvous:
            session_rendezvous['nb_columns'] = {}
        session_rendezvous['nb_columns'][uid] = self.getNbColumns() + self.NB_COLUMNS
        self.request.SESSION['rendezvous'] = session_rendezvous

    def getPropositionsByOrderedDates(self):
        return RDV_RendezVousUtility.getPropositionsByOrderedDates(self)

    def saveChanges(self):
        context = self.context.aq_inner
        propositions_by_dates = {}
        for date, propositions in self.request.form.items():
            if date in ('finish', 'extend'):
                continue
            propositions_by_dates[date] = [p for p in propositions if p]
            if not propositions_by_dates[date]:
                propositions_by_dates[date] = ['']
        # save in the session only
        RDV_RendezVousUtility.setPropositionsByDates(self, propositions_by_dates)
        if 'finish' in self.request.form:
            # save to the filesystem
            context.setPropositionsByDates(propositions_by_dates)
            self.request.response.redirect(context.absolute_url())
        elif 'extend' in self.request.form:
            self.incNbColumns()
            self.request.response.redirect(context.absolute_url() + '/@@edit_propositions')



    def __init__(self, context, request):
        super(RDV_PropositionsEdit, self).__init__(context, request)


##code-section module-footer #fill in your manual code here
##/code-section module-footer





# -*- coding: utf-8 -*-
#
# File: RDV_RendezVousUtility.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

##code-section module-header #fill in your manual code here
from Acquisition import aq_inner
import copy
##/code-section module-header

from zope.interface import implements

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class RDV_RendezVousUtility:
    """
    """

    ##code-section class-header_RDV_RendezVousUtility #fill in your manual code here
    ##/code-section class-header_RDV_RendezVousUtility

    def getSelectedDates(view):
        """Return selected dates as list
        """
        context = aq_inner(view.context)
        request = view.request
        ctx_selected_dates = request.SESSION.get('rendezvous', {})
        uid = context.UID()
        if uid in ctx_selected_dates:
            return ctx_selected_dates[uid].keys()
        else:
            return context.getPropositionsByDates().keys()

    getSelectedDates = staticmethod(getSelectedDates)
    def toggleSelectedDate(view, selected_date):
        """Toogle selected date
        """
        context = aq_inner(view.context)
        request = view.request
        uid = context.UID()
        selected_dates = RDV_RendezVousUtility.getPropositionsByDates(view)
        if not selected_date in selected_dates:
            selected_dates[selected_date] = []
        else:
            del selected_dates[selected_date]
        # save to session
        RDV_RendezVousUtility.setPropositionsByDates(view, selected_dates)

    toggleSelectedDate = staticmethod(toggleSelectedDate)
    def setPropositionsByDates(view, selected_dates):
        context = aq_inner(view.context)
        request = view.request
        uid = context.UID()
        session_rendezvous = request.SESSION.get('rendezvous', {})
        session_rendezvous[uid] = selected_dates
        request.SESSION['rendezvous'] = session_rendezvous

    setPropositionsByDates = staticmethod(setPropositionsByDates)
    def getPropositionsByDates(view):
        """Return a copy of selected dates with propositions
        """
        context = aq_inner(view.context)
        request = view.request
        ctx_selected_dates = request.SESSION.get('rendezvous', {})
        uid = context.UID()
        if uid in ctx_selected_dates:
            return copy.deepcopy(ctx_selected_dates[uid])
        else:
            return context.getPropositionsByDates()

    getPropositionsByDates = staticmethod(getPropositionsByDates)
    def getPropositionsByOrderedDates(view):
        propositions_by_dates = RDV_RendezVousUtility.getPropositionsByDates(view)
        return [(date, propositions_by_dates[date]) for date in sorted(propositions_by_dates.keys())]

    getPropositionsByOrderedDates = staticmethod(getPropositionsByOrderedDates)

##code-section module-footer #fill in your manual code here
##/code-section module-footer





# -*- coding: utf-8 -*-
#
# File: RDV_RendezVous.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.rendezvous.config import *

##code-section module-header #fill in your manual code here
from Products.CMFCore.utils import getToolByName
import md5
##/code-section module-header

schema = Schema((

    LinesField(
        name='participants',
        widget=LinesField._properties['widget'](
            label="Participants",
            label_msgid="rendezvous_RendezVous_participants_label",
            description="List of members who have participated",
            description_msgid="rendezvous_RendezVous_participants_description",
            visible=-1,
            i18n_domain='rendezvous',
        ),
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

RDV_RendezVous_schema = OrderedBaseFolderSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class RDV_RendezVous(OrderedBaseFolder, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()

    implements(interfaces.IRDV_RendezVous)

    meta_type = 'RDV_RendezVous'
    _at_rename_after_creation = True

    schema = RDV_RendezVous_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    security.declarePublic('getMembers')
    def getMembers(self):
        """
        """
        pass

    security.declarePublic('getPropositionsByDates')
    def getPropositionsByDates(self):
        """Return propositions by dates
        """
        propositions_by_dates = {}
        for date, propositions in self.getPropositionObjectsByDates().items():
            propositions_by_dates[date] = []
            propositions_ = propositions_by_dates[date]
            for proposition in propositions:
                propositions_.append(proposition[0])
        return propositions_by_dates

    security.declarePublic('getPropositionObjectsByDates')
    def getPropositionObjectsByDates(self):
        """Return propositions by dates, example:
           {'2008-11-27': [('evening': 'prop-2008-11-27-md5hexdigest')]}
        """
        propositions_by_dates = {}
        for proposition in self.objectValues("RDV_Proposition"):
            datetime = proposition.getDate()
            date = '%0.4d-%0.2d-%0.2d' % (datetime.year(), datetime.month(), datetime.day())
            if not date in propositions_by_dates:
                propositions_by_dates[date] = []
            # we use a list of items instead of dict to keep order
            propositions_by_dates[date].append((proposition.Title(), proposition.getId()))
        return propositions_by_dates

    security.declarePublic('setPropositionsByDates')
    def setPropositionsByDates(self, propositions_by_dates):
        """Create or update RDV_Proposition objects contained in this RDV_RendezVous
        """
        proposition_objects_by_dates = self.getPropositionObjectsByDates()
        portal_workflow = getToolByName(self, "portal_workflow")
        for date in sorted(propositions_by_dates.keys()):
            propositions = propositions_by_dates[date]
            for proposition in propositions:
                if date not in proposition_objects_by_dates or proposition not in dict(proposition_objects_by_dates[date]):
                    prop_id = self.invokeFactory('RDV_Proposition', 'prop-%s-%s' % (date, md5.md5(proposition).hexdigest()))
                    prop = getattr(self, prop_id)
                    prop.setTitle(proposition)
                    prop.setDate(date)
                    portal_workflow.doActionFor(prop, 'show')
                #else: we do nothing because proposition already exist
        # now delete objects not used
        objects2delete = []
        for date, propositions in proposition_objects_by_dates.items():
            if date not in propositions_by_dates:
                objects2delete.extend([p for p in dict(propositions).values()])
            else:
                for proposition in dict(propositions):
                    if proposition not in propositions_by_dates[date]:
                        objects2delete.append(dict(proposition_objects_by_dates[date])[proposition])
        if objects2delete:
            self.manage_delObjects(objects2delete)

    security.declarePublic('addParticipant')
    def addParticipant(self, member_id):
        """Add member_id to the participants list
        """
        if member_id in self.getParticipants():
            return
        participants = list(self.getParticipants())
        participants.append(member_id)
        self.setParticipants(participants)


registerType(RDV_RendezVous, PROJECTNAME)
# end of class RDV_RendezVous

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: testRDV_RendezVous.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

##code-section module-header #fill in your manual code here
##/code-section module-header

#
# Test-cases for class(es) 
#

from Testing import ZopeTestCase
from Products.rendezvous.config import *
from Products.rendezvous.tests.testPlone import testPlone

# Import the tested classes
from Products.rendezvous.content.RDV_RendezVous import RDV_RendezVous

##code-section module-beforeclass #fill in your manual code here
from Products.rendezvous.browser.RDV_RendezVousEdit import RDV_RendezVousEdit
from Products.rendezvous.browser.RDV_PropositionsEdit import RDV_PropositionsEdit
from zope.publisher.browser import TestRequest
from zope.component import getMultiAdapter
##/code-section module-beforeclass


class testRDV_RendezVous(testPlone):
    """Test-cases for class(es) ."""

    ##code-section class-header_testRDV_RendezVous #fill in your manual code here
    ##/code-section class-header_testRDV_RendezVous

    def afterSetUp(self):
        self.app.REQUEST.SESSION = {}
        self.setRoles('Manager')
        rdv_id = self.portal.invokeFactory('RDV_RendezVous', 'rdv1')
        self.rdv = getattr(self.portal, rdv_id)
        self.portal.portal_membership.addMember("johndoe", "passwd", ['Member'], [], properties={'name': 'John Doe', 'email': 'johndoe@example.com'})
        self.johndoe = self.portal.portal_membership.getMemberById('johndoe')

    # Manually created methods

    def test_DelDateWithPropositions(self):
        propositions = self._create_propositions()
        # remove one date
        expected_propositions = {'2008-11-28':['8:30'],
                                 '2008-11-29':[''],
        }
        self.rdv.setPropositionsByDates(expected_propositions)
        self.assertEquals(expected_propositions, self.rdv.getPropositionsByDates())

    def test_AddDate(self):
        request = self.app.REQUEST
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals([], view.getSelectedDates())
        request.set('rdvdate', '2008-11-27')
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(['2008-11-27'], view.getSelectedDates())
        request.set('rdvdate', '2008-11-28')
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(['2008-11-27', '2008-11-28'], view.getSelectedDates())
        request.set('rdvdate', '2008-11-29')
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(['2008-11-27', '2008-11-28', '2008-11-29'], view.getSelectedDates())

    def test_manage_participants_in_proposition(self):
        prop_id = self.rdv.invokeFactory('RDV_Proposition', 'prop')
        prop = getattr(self.rdv, prop_id)
        prop.manageParticipant('johndoe', checked=True)
        self.assertEquals(('johndoe',), prop.getAvailable())
        prop.manageParticipant('johnsmith', True)
        self.assertEquals(('johndoe', 'johnsmith'), prop.getAvailable())
        prop.manageParticipant('johndoe', False)
        self.assertEquals(('johnsmith',), prop.getAvailable())

    def test_SaveChanges(self):
        request = self.app.REQUEST
        view = RDV_PropositionsEdit(self.rdv, request)
        props = {'2008-12-25': ['12:00', 'minuit', 'midi', '', ''],
                 '2008-12-19': ['13:00', '14:00', '', '', ''],
                 '2008-12-31': ['', '', '', '', ''] }
        request.form = props
        request.form['finish'] = 'Finish'
        view.saveChanges()
        self.assertEquals({'2008-12-25': ['12:00', 'minuit', 'midi'],
                           '2008-12-19': ['13:00', '14:00'],
                            '2008-12-31': ['']}, self.rdv.getPropositionsByDates())

    def _create_propositions(self):
        propositions = {'2008-11-27':['morning', 'evening'],
                        '2008-11-28':['8:30'],
                        '2008-11-29':[''],
        }

        self.rdv.setPropositionsByDates(propositions)
        return propositions

    def test_SetPropositions(self):
        propositions = self._create_propositions()
        self.assertEquals(propositions, self.rdv.getPropositionsByDates())

    def test_reedit_rendezvous(self):
        propositions = self._create_propositions()
        request = self.app.REQUEST
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(propositions.keys(), view.getSelectedDates())

    def test_NbColumns(self):
        request = self.app.REQUEST
        view = RDV_PropositionsEdit(self.rdv, request)
        self.assertEquals(view.NB_COLUMNS, view.getNbColumns())

    def test_incColumns(self):
        request = self.app.REQUEST
        request.form['extend'] = 'Extends columns'
        view = RDV_PropositionsEdit(self.rdv, request)
        self.assertEquals(view.NB_COLUMNS, view.getNbColumns())
        view.saveChanges()
#        view.incNbColumns()
        self.assertEquals(view.NB_COLUMNS*2, view.getNbColumns())

    def test_DelDate(self):
        request = self.app.REQUEST
        # add two dates
        request.set('rdvdate', '2008-11-27')
        view = RDV_RendezVousEdit(self.rdv, request)
        request.set('rdvdate', '2008-11-28')
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(['2008-11-27', '2008-11-28'], view.getSelectedDates())
        # remove one date
        request.set('rdvdate', '2008-11-27')
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(['2008-11-28'], view.getSelectedDates())

    def test_add_other_dates(self):
        propositions = self._create_propositions()
        request = self.app.REQUEST
        request.set('rdvdate', '2008-11-30')
        view = RDV_RendezVousEdit(self.rdv, request)
        self.assertEquals(sorted(list(propositions.keys()) + ['2008-11-30']), sorted(view.getSelectedDates()))

    def test_add_participant(self):
        self.rdv.addParticipant('johndoe')
        self.assertEquals(('johndoe',), self.rdv.getParticipants())

    def test_add_participant_twice(self):
        self.rdv.addParticipant('johndoe')
        self.rdv.addParticipant('johndoe')
        self.assertEquals(('johndoe',), self.rdv.getParticipants())

    def test_addParticipant(self):
        request = self.app.REQUEST
        view = RDV_PropositionsEdit(self.rdv, request)
        props = {'2008-12-25': ['12:00', 'minuit', 'midi', '', ''],
                 '2008-12-19': ['13:00', '14:00', '', '', ''],
                 '2008-12-31': ['', '', '', '', ''] }
        request.form = props
        request.form['finish'] = 'Finish'
        view.saveChanges()
        request = TestRequest()
        request.AUTHENTICATED_USER = self.johndoe
        propositions = self.rdv.getPropositionObjectsByDates()
        request.form = {'propositions': (propositions['2008-12-25'][0][1],)}
        view = getMultiAdapter((self.rdv, request), name='add-participation')
        view()
        prop = getattr(self.rdv, propositions['2008-12-25'][0][1])
        self.assertEquals(prop.getAvailable(), ('johndoe',))
        self.assertEquals(prop.getUnavailable(), ())

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testRDV_RendezVous))
    return suite

##code-section module-footer #fill in your manual code here
##/code-section module-footer

if __name__ == '__main__':
    framework()





import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.1.1'

long_description = (
    read('README.txt')
    + '\n' +
    read('docs', 'HISTORY.txt')
    + '\n' +
    'Download\n'
    '========\n'
    )

setup(name='Products.rendezvous',
      version=version,
      description="A timeboard to select a rendez-vous for Plone",
      long_description=long_description,
      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: User Interfaces",
        "Framework :: Plone",
        ],
      keywords='',
      author='Vincent Fretin and Michael Launay',
      author_email='development@ecreall.com',
      url='http://pypi.python.org/pypi/Products.rendezvous',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
#
# File: RDV_RendezVousView.py
#
# Copyright (c) 2008 by Ecreall
# Generator: ArchGenXML Version 2.2 (svn)
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Vincent Fretin <vincentfretin@ecreall.com>"""
__docformat__ = 'plaintext'

##code-section module-header #fill in your manual code here
from Products.rendezvous.browser.RDV_RendezVousUtility import RDV_RendezVousUtility
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from Products.CMFCore.interfaces import ISiteRoot
##/code-section module-header

from zope import interface
from zope import component
from Products.CMFPlone import utils
from Products.Five import BrowserView
from zope.interface import implements
from Products.rendezvous.content.RDV_RendezVous import RDV_RendezVous
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class RDV_RendezVousView(BrowserView):
    """
    """

    ##code-section class-header_RDV_RendezVousView #fill in your manual code here
    AVAILABLE = "available"
    UNAVAILABLE = "unavailable"
    UNKNOWN = "unknown"
    ##/code-section class-header_RDV_RendezVousView

    @memoize
    def getPropositionsItemsByOrderedDates(self):
        context = self.context.aq_inner
        propositions_by_dates = context.getPropositionObjectsByDates()
        return [(date, propositions_by_dates[date]) for date in sorted(propositions_by_dates.keys())]

    def getParticipationClass(self, participant, proposition):
        prop_obj = getattr(self.context, proposition)
        if participant in prop_obj.getAvailable():
            return self.AVAILABLE
        if participant in prop_obj.getUnavailable():
            return self.UNAVAILABLE
        return self.UNKNOWN

    def addParticipation(self):
        context = self.context.aq_inner
        actor = self.request.AUTHENTICATED_USER
        participant = actor.getId()
        try:
            selected_propositions = self.request.form['propositions']
        except KeyError:
            selected_propositions = ()
        context.addParticipant(participant)
        for prop in self._getAllPropositionsIds():
            prop_obj = getattr(context, prop)
            checked = prop in selected_propositions
            prop_obj.manageParticipant(participant, checked)

        actor_fullname = actor.getProperty('fullname', participant)
        actor_email = actor.getProperty('email', None)
        encoding = getUtility(ISiteRoot).getProperty('email_charset')
        template = getattr(self.context, "rdv_participation_notification")
        owner = self.context.getOwner()
        owner_email = owner.getProperty('email', None)
        owner_name = owner.getProperty('fullname', owner.getId())
        if owner_email:
            message = template(self.context, self.request,
                                       actor_fullname=actor_fullname,
                                       actor_email=actor_email,
                                       receipt_to_email=owner_email,
                                       receipt_to_name=owner_name)
            self.context.MailHost.send(message.encode(encoding))

        self.request.response.redirect(self.context.absolute_url())

    def getFullname(self, member_id):
        portal_membership = getToolByName(self, "portal_membership")
        member = portal_membership.getMemberById(member_id)
        return member and member.getProperty("fullname") or member_id

    def getNbParticipantsForProposition(self, prop_id):
        return len(getattr(self.context, prop_id).getAvailable())

    def _getAllPropositionsIds(self):
        return [p[1] for date, propositions in self.getPropositionsItemsByOrderedDates()
                         for p in propositions]

##code-section module-footer #fill in your manual code here
##/code-section module-footer


